/*
  The Knob
    
  Program provides for a rotary dial to control a websdr website frequency tuning. On websdr.org you can find 
  a list of available websdr receivers all over the world. Select one and enable keyboard control. This sketch 
  uses the Keyboard library. The arduino is seen as a keyboard to the computer. Warning: the keyboard library 
  only works on some arduino boards (with native USB) such as Arduino Leonardo, Micro or Due. This version is 
  written for Pro Micro 5V and an optical rotary encoder. Mechanical encoders will most likely nog work due to
  contact bouncing. Device is directly powerd from the USB bus. 

Pin allocation:
  2 encoder A (enc_A) external pull-up 1k to 5 Volt --> Must be an interrupt pin!!!!!
  3 encoder B (enc_B) external pull-up 1k to 5 Volt
  4 Front LED (frontLed) (a high output switches the led on) --> appropriate series resistor. 
  5 Switch to enable or disable character sending
  
15 September 2020
by Joris PE1GLX

This code is in the public domain. Use of this code is at your own risk.  
Info on the used library can be found at 
 http://www.arduino.cc/en/Tutorial/KeyboardSerial
*/

#define enc_A (2)            // pin number encoder input A (HW pullup 1k) --> Must be an interrupt pin!!!!!
#define enc_B (3)            // pin number encoder input B (HW pullup 5k6//1k)
#define frontLed (4)         // pin number of the LED on the front
#define swL (5)              // switch 
#define p_skip (1)          //pulses to skip
#define m_skip (30)         // multiplier to calculate b_skip=(p_skip+1)*m_skip
#define chardelay (5)       // delay between send charaters [ms]
bool enable=LOW;
int b_skip;                 //pulses to skip for a big step
int enc_UD=0;
#define enc_dir (-1)        // can be 1 or -1 to set encoder direction correct
#include "Keyboard.h"

//**********************************setup*********************************

void setup() {
  pinMode(enc_A,INPUT);
  pinMode(enc_B,INPUT);
  pinMode(frontLed,OUTPUT);
  pinMode(swL,INPUT_PULLUP);
  Keyboard.begin();    // initialize control over the keyboard:
  b_skip=(p_skip+1)*m_skip;
  attachInterrupt(digitalPinToInterrupt(enc_A), encoderInt, RISING);
}

//******************************interupt routine encoderInt**************
void encoderInt() {
  if (digitalRead(enc_B)) enc_UD+=enc_dir;
  else enc_UD+=-enc_dir;
}


//******************************main loop*******************************
void loop() {
  if (digitalRead(swL)==LOW){enable=HIGH;}
  else {enable=LOW;} digitalWrite(frontLed,enable);  
  
  delay(chardelay);
  
  if (enable){
    if (enc_UD>+b_skip){enc_UD-=b_skip; Keyboard.press(129); Keyboard.write(215); Keyboard.releaseAll();}
    else if (enc_UD<-b_skip){enc_UD+=b_skip; Keyboard.press(129); Keyboard.write(216); Keyboard.releaseAll();}
    else if (enc_UD>+p_skip){enc_UD-=p_skip; Keyboard.write(215);}
    else if (enc_UD<0){enc_UD+=p_skip; Keyboard.write(216);} 
    else {}  
  }
  else {enc_UD=0;}  //
}
